mix firmware (nerves v2.0.0-pre.2)

Copy Markdown View Source

Build a firmware image for the selected target platform.

This is a convenience wrapper around mix release. The firmware image is created as a release step, so mix release produces the same .fw file when the project is configured with Nerves.BuildAction actions.

Command line options

  • --verbose - produce detailed output about release assembly

Environment variables

  • NERVES_SYSTEM - may be set to a local directory to specify the Nerves system image that is used

  • NERVES_TOOLCHAIN - may be set to a local directory to specify the Nerves toolchain (C/C++ cross-compiler) that is used