mix nerves.discover (nerves v2.0.0-pre.2)

Copy Markdown View Source

Discover Nerves devices on the local network using mDNS

This task scans the local network for Nerves devices and displays their information in a table format.

Linux users: Install avahi-utils for faster responses.

Examples

$ mix nerves.discover

$ mix nerves.discover --timeout 10000

Options

  • --timeout - timeout in milliseconds to wait for replies (default 5000)