Delete Nerves artifact files and container build volumes
By default, deletes only the artifacts matching the current fingerprint of
the specified package. Pass --all to every build of the artifact.
Shows what will be deleted and asks for confirmation before proceeding.
MIX_TARGET must be set so that target-specific dependencies are
available. When no package name is given, the task auto-selects if
there is exactly one Nerves artifact dependency.
Command line options
--all- Delete all versions, not just the current one--yes- Skip the confirmation prompt
Examples
$ MIX_TARGET=rpi0 mix nerves.artifact.purge
$ MIX_TARGET=rpi0 mix nerves.artifact.purge test_system_rpi0
$ MIX_TARGET=rpi0 mix nerves.artifact.purge --all test_system_rpi0
$ MIX_TARGET=rpi0 mix nerves.artifact.purge --yes test_system_rpi0