Utilities for figuring out compilation target tuples
Fun rant: https://mcyoung.xyz/2025/04/14/target-triples/
Standardizing on target tuple contents is hard so this module exists to convert tuples to whatever gets the job done. The pain is intended to be here, so that workarounds don't propagate everywhere.
Summary
Functions
Convert a target tuple back to it's :erlang.system_info(:system_architecture) form
Create a new tuple
Convert a host tuple into the tuple format used by Nerves v1 toolchain artifacts
Types
Functions
Convert a target tuple back to it's :erlang.system_info(:system_architecture) form
Create a new tuple
If an argument isn't passed, the host is queried.
Convert a host tuple into the tuple format used by Nerves v1 toolchain artifacts